#version 120

uniform float blindness;
uniform float centerDepthSmooth;
uniform float far;
uniform float frameTimeCounter;
uniform float rainStrength;
uniform float wetness;
uniform int heldBlockLightValue;
uniform int heldItemId;
uniform int isEyeInWater;
uniform int moonPhase;
uniform int worldDay;
uniform int worldTime;
uniform ivec2 eyeBrightnessSmooth;
uniform mat4 gbufferModelView;
uniform mat4 gbufferModelViewInverse;
uniform mat4 gbufferProjection;
uniform mat4 gbufferProjectionInverse;
uniform vec3 cameraPosition;
        vec3 eyePosition = cameraPosition + vec3(0.0, gbufferModelViewInverse[3][1], 0.0);
uniform vec3 fogColor;
uniform vec3 skyColor;
uniform vec3 sunPosition;
uniform vec3 upPosition;

#define BLUR_ENABLED //Is blur enabled at all?
//#define DOF_BLUR //Blurs things that are at a different distance than whatever's in the center of your screen

#define DYNAMIC_LIGHTS //Holding blocks that emit light will light up their surroundings
#define HARDCORE_DARKNESS 0 //0 (Off): Normal visibility at night. 1 (On): Complete darkness at night. 2 (Moon phase) Nighttime brightness is determined by the current phase of the moon. [0 1 2]
#define EYE_ADJUST //Allows your eyes to "adjust" to darkness
#define CLOUDS //3D clouds (partially volumetric too). Mild performance impact!
//#define OLD_CLOUDS //Uses old cloud rendering method from earlier versions, for people who don't like pretty things.
#define CLOUD_NORMALS //Dynamically light clouds based on weather they're facing towards or away from the sun. Mild performance impact!
#define CLOUD_HEIGHT 256.0 //Y level of fancy clouds [128.0 144.0 160.0 176.0 192.0 208.0 224.0 240.0 256.0 272.0 288.0 304.0 320.0 336.0 352.0 368.0 384.0 400.0 416.0 432.0 448.0 464.0 480.0 496.0 512.0]

#define SUNSET_COEFFICIENT_RED 7.2 //Red sunset coefficient. Higher values will result in the red color of sunset starting earlier and persisting longer. A change of 0.1 corresponds to about 5 seconds real-world time. [6.0 6.1 6.2 6.3 6.4 6.5 6.6 6.7 6.8 6.9 7.0 7.1 7.2 7.3 7.4 7.5 7.6 7.7 7.8 7.9 8.0]
#define SUNSET_COEFFICIENT_GREEN 6.8 //Green sunset coefficient. Higher values will result in the green color of sunset starting earlier and persisting longer. A change of 0.1 corresponds to about 5 seconds real-world time. [6.0 6.1 6.2 6.3 6.4 6.5 6.6 6.7 6.8 6.9 7.0 7.1 7.2 7.3 7.4 7.5 7.6 7.7 7.8 7.9 8.0]
#define SUNSET_COEFFICIENT_BLUE 6.2 //Blue sunset coefficient. Higher values will result in the blue color of sunset starting earlier and persisting longer. A change of 0.1 corresponds to about 5 seconds real-world time. [6.0 6.1 6.2 6.3 6.4 6.5 6.6 6.7 6.8 6.9 7.0 7.1 7.2 7.3 7.4 7.5 7.6 7.7 7.8 7.9 8.0]

varying vec3 BLOCK_LIGHT_COLOR; //Color of block light. Gets yellow-er if you stay away from light-emitting blocks for a while.
varying vec3 SHADOW_COLOR; //Color of shadows. Sky-colored, to simulate indirect lighting.
varying vec3 SKY_LIGHT_COLOR; //Color of sky light. Is usually white during the day, and very dark blue at night.
varying vec4 HELD_LIGHT_COLOR; //Color of held light source. Alpha = brightness.

#include "/lib/noisetex.glsl"

varying vec2 texcoord;
varying vec3 sunPosNorm;
varying vec3 upPosNorm;

varying float adjustedTime; //More useful number than worldTime.
varying float day; //1.0 during the day, 0.0 at night.
varying float night; //1.0 at night, 0.0 during the day.
varying float sunset; //1.0 at sunset (when day and night are both < 1.0).
varying float phase; //More useful number than moonPhase.

varying float eyeAdjust; //How much brighter to make the world

#ifdef CLOUDS
	varying float CLOUD_DENSITY_MODIFIER; //Random fluctuations every few minutes.
	varying vec3 CLOUD_COLOR; //Color of the side of clouds facing away from the sun.
	varying vec3 CLOUD_ILLUMINATION_COLOR; //Color of the side of clouds facing towards the sun.
	varying vec4 CLOUD_INSIDE_COLOR; //Color to render over your entire screen when inside a cloud.
#endif

#if defined(BLUR_ENABLED) && defined(DOF_BLUR)
	varying float dofDistance; //Un-projected centerDepthSmooth
#endif

#include "/lib/math.glsl"

#include "/lib/calcHeldLightColor.glsl"

#include "/lib/goldenOffsets.glsl"

#ifdef CLOUDS
	#ifdef OLD_CLOUDS
		#include "/lib/drawClouds_old.glsl"
	#else
		#include "/lib/drawClouds.glsl"
	#endif
#endif

void main() {
	gl_Position = ftransform();
	texcoord = (gl_TextureMatrix[0] * gl_MultiTexCoord0).xy;

	sunPosNorm = normalize(sunPosition);
	upPosNorm = normalize(upPosition);

	adjustedTime = abs(mod(worldTime / 1000.0 + 6.0, 24.0) - 12.0);
	day = clamp(6.7 - adjustedTime, 0.0, 1.0);
	night = clamp(adjustedTime - 6.7, 0.0, 1.0);
	sunset = 1.0 - day - night;
	phase = abs(float(moonPhase) * 0.25 - 1.0);

	#ifdef EYE_ADJUST
		eyeAdjust = 3.0 - 1.5 * max(eyeBrightnessSmooth.x / 240.0, (isEyeInWater == 1 ? eyeBrightnessSmooth.y / 480.0 + 0.5 : eyeBrightnessSmooth.y / 240.0) * (1.0 - night));
	#else
		eyeAdjust = 1.5;
	#endif

	#if defined(BLUR_ENABLED) && defined(DOF_BLUR)
		vec4 v = gbufferProjectionInverse * vec4(0.0, 0.0, centerDepthSmooth * 2.0 - 1.0, 1.0);
		dofDistance = -v.z / v.w;
	#endif

	#ifdef CLOUDS
		//avoid some floating point precision errors on old worlds by modulus-ing the world day.
		//would just do the modulo on ints instead of floats, but if I've learned anything about GLSL, it's that it really doesn't like ints.
		//as such, I'll do it float-wise instead in hopes that it won't randomly break on someone else's GPU.
		float randTime = (mod(float(worldDay), float(noiseTextureResolution)) + worldTime / 24000.0) * 5.0;
		randTime = floor(randTime) + interpolateSmooth1(fract(randTime)) + 0.5;
		CLOUD_DENSITY_MODIFIER = (texture2D(noisetex, vec2(randTime, 0.5) * invNoiseRes).r * 3.0 - 1.5) * (1.0 - wetness);
		//float r0 = texelFetch2D(noisetex, ivec2(int(randTime)     % noiseTextureResolution, 0), 0).r;
		//float r1 = texelFetch2D(noisetex, ivec2(int(randTime + 1) % noiseTextureResolution, 0), 0).r;
		//CLOUD_DENSITY_MODIFIER = (mix(r0, r1, interpolateSmooth1(fract(randTime))) * 3.0 - 1.5) * (1.0 - wetness);

		CLOUD_COLOR              = mix(vec3(0.48, 0.5, 0.55), fogColor * 0.6, /*vec3(0.175, 0.225, 0.3),*/ rainStrength) * day;
		CLOUD_ILLUMINATION_COLOR = mix(vec3(1.0, 1.0, 1.0), vec3(0.45, 0.5, 0.6), rainStrength) * day;

		if (sunset > 0.001) {
			vec3 sunsetColor = clamp(vec3(SUNSET_COEFFICIENT_RED + 0.2, SUNSET_COEFFICIENT_GREEN + 0.2, SUNSET_COEFFICIENT_BLUE + 0.2) - adjustedTime, 0.0, 1.0);
			CLOUD_ILLUMINATION_COLOR = mix(CLOUD_ILLUMINATION_COLOR, sunsetColor, sunset * (1.0 - rainStrength * 0.5));
		}

		float d = abs(eyePosition.y - CLOUD_HEIGHT) / 4.0;
		if (d < 1.0) {
			CLOUD_INSIDE_COLOR = drawClouds(vec3(0.0), vec3(0.0), d, true);
			if (CLOUD_INSIDE_COLOR.a > 0.001) {
				if (d > 0.001 && d < 0.999) CLOUD_INSIDE_COLOR.a *= interpolateSmooth1(d); //in the fadeout range
			}
		}
		else CLOUD_INSIDE_COLOR = vec4(0.0);
	#endif

	#include "/lib/heldlightData.glsl"

	#include "/lib/colors.glsl"
}