#version 120

uniform float rainStrength;
uniform float viewHeight;
uniform float viewWidth;
uniform float wetness;
uniform int worldDay;
uniform int worldTime;
uniform mat4 gbufferModelViewInverse;
uniform mat4 gbufferProjectionInverse;
uniform vec3 cameraPosition;
uniform vec3 fogColor;
uniform vec3 skyColor;

#define INFINITE_OCEANS //Simulates water out to the horizon instead of just your render distance.
//#define CROSS_PROCESS //Opposite of desaturation, makes everything more vibrant and saturated.

#define FANCY_STARS //Improved stars in the overworld
#define GALAXIES //Galaxies visible at night in the overworld, with even more stars inside them
#define EXCLUSION_RADIUS 1.0 //Radius around the moon at which fancy stars/galaxies stop rendering [0.5 0.6 0.7 0.8 0.9 1.0 1.1 1.2 1.3 1.4 1.5 1.6 1.7 1.8 1.9 2.0]
//#define CUBIC_CHUNKS //Disables black fog/sky colors below Y=0

#define SUNSET_COEFFICIENT_RED 7.2 //Red sunset coefficient. Higher values will result in the red color of sunset starting earlier and persisting longer. A change of 0.1 corresponds to about 5 seconds real-world time. [6.0 6.1 6.2 6.3 6.4 6.5 6.6 6.7 6.8 6.9 7.0 7.1 7.2 7.3 7.4 7.5 7.6 7.7 7.8 7.9 8.0]
#define SUNSET_COEFFICIENT_GREEN 6.8 //Green sunset coefficient. Higher values will result in the green color of sunset starting earlier and persisting longer. A change of 0.1 corresponds to about 5 seconds real-world time. [6.0 6.1 6.2 6.3 6.4 6.5 6.6 6.7 6.8 6.9 7.0 7.1 7.2 7.3 7.4 7.5 7.6 7.7 7.8 7.9 8.0]
#define SUNSET_COEFFICIENT_BLUE 6.2 //Blue sunset coefficient. Higher values will result in the blue color of sunset starting earlier and persisting longer. A change of 0.1 corresponds to about 5 seconds real-world time. [6.0 6.1 6.2 6.3 6.4 6.5 6.6 6.7 6.8 6.9 7.0 7.1 7.2 7.3 7.4 7.5 7.6 7.7 7.8 7.9 8.0]

#include "/lib/constants.glsl"

varying float adjustedTime; //More useful number than worldTime.
varying float day; //1.0 during the day, 0.0 at night.
varying float night; //1.0 at night, 0.0 during the day.
varying float sunset; //1.0 at sunset (when day and night are both < 1.0).
varying float phase; //More useful number than moonPhase.

varying vec3 sunPosNorm;
varying vec3 upPosNorm;

varying vec4 starData; //rgb = star color, a = flag for weather or not this pixel is a star.

const float sunPathRotation = 30.0; //Angle that the sun/moon rotate at [-45.0 -40.0 -35.0 -30.0 -25.0 -20.0 -15.0 -10.0 -5.0 0.0 5.0 10.0 15.0 20.0 25.0 30.0 35.0 40.0 45.0]
#if defined(FANCY_STARS) || defined(GALAXIES)
	const mat2 starRotation = mat2(
		cos(sunPathRotation * 0.01745329251994), -sin(sunPathRotation * 0.01745329251994),
		sin(sunPathRotation * 0.01745329251994),  cos(sunPathRotation * 0.01745329251994)
	);
#endif

#include "/lib/noisetex.glsl"

#include "/lib/math.glsl"

#include "/lib/hue.glsl"

#include "/lib/drawStars.glsl"

#include "/lib/calcSkyColorFull.glsl"

//checks a few conditions before actually calculating the sky color.
vec4 checkSkyColor(vec3 pos) {
	#ifdef INFINITE_OCEANS
		float upDot = dot(pos, upPosNorm);
		if (upDot < 0.0) return vec4(0.0); //calculated in composite instead.
	#endif

	if (starData.a > 0.9) {
		#ifdef FANCY_STARS
			return vec4(0.0, 0.0, 0.0, 1.0);
		#else
			#ifdef INFINITE_OCEANS
				return vec4(starData.rgb * (1.0 - fogify(upDot * square(max(cameraPosition.y, 256.0) / 256.0 + 1.0), 0.25)), 1.0); //apply fog to stars near the horizon
			#else
				return starData;
			#endif
		#endif
	}

	return vec4(calcSkyColor(pos), 1.0);
}

void main() {
	vec3 pos = normalize((gbufferProjectionInverse * vec4(gl_FragCoord.xy / vec2(viewWidth, viewHeight) * 2.0 - 1.0, 1.0, 1.0)).xyz);
	vec4 color = checkSkyColor(pos);

/* DRAWBUFFERS:04 */
	gl_FragData[0] = color; //gcolor
	gl_FragData[1] = vec4(0.0, 0.0, 0.0, color.a * 0.5); //gaux1
}